/*****************************************************************************
*
*  Radio Communications code borrowed and ported to PIC from:
*  WIRELESS MOUSETRAP MONITORING SYSTEM by Alberto Ricci Bitti (c) 2003
*  www.riccibitti.com
*  Original Article:
*  http://www.circuitcellar.com/library/print/0604/RicciBitti-167/RicciBitti-167.pdf
*  ftp.circuitcellar.com/pub/Circuit_Cellar/2004/167
*
*****************************************************************************
*
*  MODULE:   RXM_TX684_MAIN.C
*  PURPOSE:  Alarm Monitoring System - Alarm transmitter main app.
*  REQUIRES: No special requirements.
*  NOTES:    Imported code ported from MC68HC908QT chip to PIC12F683
*  Compiler: CCS PCM V3.236 or later (V4107)
*
*****************************************************************************
*
*
*     Revision List:
*
*        Ver      Date        Who      Description
*        ---------------------------------------------------------------------
*        X1.00    06.07.31    GJO      Wrote test program for LINX TX using 16F627
*        V1.00    06.07.19    GJO      Begin application integration
*        V1.01    08.05.19    GJO      Added 8-bit Internet Checksum and increased
*                                       packet size from 4 to 6 bytes
*
*       V1.00   08.06.26    GJO         Begin adaptation to new F684 board
*               10.07.02    GJO         Reviewed source for further development with V4107
*
*
******************************************************************************/

//#define LM75_PRESENT

#include "D101A.h"
#include "D101A_HW.c"

/******************************************************************************/
/* RF packet Data Format                                                      */
/******************************************************************************/

//              OUTPUT WAVEFORM TO RF TRANSMITTER
//
//        logic 1 = .66 mS .33 mS
//                  +-----+--+
//
//
//                logic 0 = .33 mS .66 mS
//                           +--+-----+
//        |         |        |        |        |
//
//               +--+     +--+  +--+--+     +--+     +--
//               |  |     |  |  |     |     |  |     |
//----....-------+  +--+--+  +--+     +--+--+  +--+--+
//
//
//  sync pause   start    1' bit   2'bit    3'bit     ... total: 23 bits + start
//
//   (11 mS minimum)
//
//
//   Bit   Fields for RF packet:
//   ==============================|===========================================
//   0   1   2   3   4   5   6    7    8    9    10   11   12   13   14   15
//   |   |   |   |   |            |    |    |    |    |     |              |
//   SB  C2  C1  C0  +-- Seq No --+    + --KTY MSB -- +     +  Source ID   +
//   ==================================|=======================================
//   16  17  18  19  20  21  22   23   24   25   26   27   28   29   30   31
//   |   |   |   |   |   |   | MSB                                     LSB |
//   S3  S2  S1  S0  AS0 AS1 |----------------  BATTERY VOLTS -------------+
//   ================================|=========================================
//   32  33  34  35  36  37  38   39   40   41   42   43   44   45   46   47
//   MSB                             /                LSB  MSB
//   | -------------- EXTERNAL VOLTS --------------------| | ----- INTERNAL
//   ================================|=========================================
//   48  49  50  51  52  53  54   55   56   57   58   59   60   61   62   63
//                             LSB
//   TEMPERATURE ------------------|   |------------ Checksum -------------|
//   ==========================================================================
//
//
//
//   Bits         Name   Description
//  ===========================================================================
//   0             SB   -   Start Bit
//   01            C2   -   SF
//   02            C1   -   SF
//   03            C0   -   SF
//   04-07         seq  -   4 Bit Frame Sequence Nr.
//   08-11         nd   -   MSB 15 to 12 of KTY81 ADC channel
//   12-15         ID   -   4 bit Source ID value
//   16            S0   -   Switch Input 1 - J1-
//   17            S1   -   Switch Input 2 - J1-
//   18            S2   -   Switch Input 3 - Not defined
//   19            S3   -   Switch Input 4 - Not defined
//   20-21         ASn  -   ADC select channel select
//   22-31  10-Bit ADC  -   Battery Volts  0-3.56V
//   32-43  10-Bit ADC  -   External Volts 0-10V
//   44-55  10-Bit ADC  -   Internal KTY81 Temperature in DEG C + offset 150
//   56-63         CS   -   Checksum byte
//
//*****************************************************************************
//*********************** BEGIN OF FUNCTIONS **********************************
//*****************************************************************************

///////////////////////////////////////////////////////////////////////////////
// get_interval - returns an ID-dependent interval (in wakeup timer ticks)
// to be used to repeat heartbeat transmissions
///////////////////////////////////////////////////////////////////////////////

static unsigned int get_interval( unsigned char ID )
{
    return ID + HEARTBEAT_SECONDS * 2; // the wakeup timer is about half a second,
}                                      // multiply seconds by 2 to get interval count


/////////////////////////////////////////////////////////////////////////
// new_transmit - assembles a codeword according to the status flags and
// ID, and transmits it many times
////////////////////////////////////////////////////////////////////////////////

void new_xmit(char reps, char *_rp)
{
    int i;
    long mask, addr;

    addr = _rp;

    TX_WAIT

    while(reps--)
    {
        for(i=0; i < PACKET_SIZE; i++)
        {
            mask = 0x80;            //prepare mask for filtering leftmost bit
            while(mask != 0x00)
            {
                TX_WAIT                    // Wait for 333us interrupt macro
                if ((*(addr + i) & mask) == 0)   //are we transmitting a zero?
                {
                    TX_START          //yes, force a 66% duty cycle
                }
                TX_WAIT               // Wait for 333us interrupt macro
                TX_START              //duty cicle must be not less than 33%
                TX_WAIT               // Wait for 333us interrupt  macro
                TX_STOP               //turn transmitter off
                mask >>= 1 ;          //prepare mask for filtering next bit
            };
        }
        // repeat rate varies with ID in order to prevent data collisions
        delay_ms( PAUSE_TIME_mS + (_rp[0] & 0x0F) % 16 );
    }
}



//*****************************************************************************
// main():
//*****************************************************************************

void main(void)
{

    char _rp[] = {0, 0, 0, 0, 0, 0, 0, 0};
    char seq=0, idnum = 0x00;
    char i;
    static long sleep_ctr = TX_INTERVALL;         // Force transmit on power up
    long sum;

    hardware_initialize();      //initialize ports, registers and interrupts

    for(;;)
    {
        if(TX_INTERVALL <= ++sleep_ctr)
        {
            ADC_ON                  // Turn On ADC power on
            SYSPWR_ON               // Turn on LM385 and system bias power
            read_ADCvolts();        // Take voltage measurements
            ADC_SD                  // Turn off AD Converter HArdware

            // Set new ID number
            if(!input(SETUP_SW)){
                write_eeprom(0,(0x0F & (idnum++ % NUM_ID)));
                for (i=0;i< read_eeprom(0) & 0x0F;i++)
                {
                    LED_ON
                    delay_ms(250);
                    LED_OFF
                    delay_ms(250);
                }
                delay_ms(1000);
                LED_ON
                delay_ms(1000);
                LED_OFF
            }

            disable_interrupts(INT_TIMER0);
            // For debug only send analogs to RS232
            putc('\n');
            for(i=0;i<4;i++)
            {
                printf("%lu ", analogs[i]);
            }
            putc('\r');
            putc('\n');
            enable_interrupts(INT_TIMER0);

            // Assemble packet data
            // To calculate temperature incorporate the ratio of R16/R17 divider
            // and adjust KTY81 divisor value
            // DEG C = {[(ADC64 * 1023) / 967] / 101} - 150
            // The term 1023/967 measures the bridge voltage difference between
            // VDD and the digital pin voltage powering the KTY81 circuit
            // Regs 1,5, and 6 are used to hold all 16 bits
            _rp[1] &= 0x0F; // Clear MSB nibble for bits 15-12
            _rp[5] &= 0xF0; // Clear LSB nibble for bits 11-8
            _rp[1] |= ((analogs[1] & 0xF000) >> 8); // BITS 15-12
            _rp[5] |= ((analogs[1] & 0x0F00) >> 8); // BITS 11-8
            _rp[6]  =  (analogs[1] & 0x00FF);       // BITS 7-0

            // Assemble External ADC Input for RP4/RP5
            // VOLTS = (ADC * VREF * 6) / 1000, where VREF = 1.780V
            _rp[5] &= 0x0F; // Clear MSB nibble
            _rp[4] = (int) ((analogs[0] & 0x03F0) >> 4);
            _rp[5] |= (int) ((analogs[0] & 0x000F) << 4);

            // Assemble 10-bit battery volts
            _rp[3] = (int) (analogs[2] & 0xFF);           // ADC LSB byte
            _rp[2] &= 0xFC;                             // Clear 2 ADC MSB bits
            _rp[2] |= (int) ((analogs[2] & 0x0300) >> 8);   // ADC MSB 2 bits

            // Assemble other bits
            _rp[2] &= 0x03;      // Unused bits must be clear, but don't touch ADC bits
            _rp[2] |= (!input(INPUT2)) ? 0x20 : 0x00;
            _rp[2] |= (!input(INPUT1)) ? 0x10 : 0x00;
            //copy non-volatile nibble value from FLASH (RF Channel ID)
            _rp[1] |= read_eeprom(0) & 0x0F;
            // Add sequence nibble to start byte
            _rp[0] = START_BIT | (0x0F & seq);

            sum=0;

            // Count Internet Checksum for radio packet bytes
            // Adapted from RFC-1171
            for(i=0;i<(PACKET_SIZE - 1);i++)
            {
                sum += (long) _rp[i];
            }

            // Fold 16-bit sum to 8 bits
            while (sum >> 8)
            {
                sum = (sum & 0xFF) + (sum >> 8);
            }

            _rp[PACKET_SIZE - 1] = (int) (~sum & 0xFF);

            LED_ON
            PDN_HIGH                // Turn on TX power
            // Transmit new data
            new_xmit( PACKET_SIZE, _rp );       // Transmit data packets
            sleep_ctr = 0;                      // Reset sleep time intervall
            seq++;                              // RF packet sequence counter
            PDN_LOW                             // Turn off TX enable and LM285
//            port_a_pullups(0);
            LED_OFF
            SYSPWR_OFF              // Turn off system bias power
        }
        // Put PIC into sleep mode, continues from here on wakeup
        sleep();
    }
}

// eof
